#include "stdafx.h"
#include <math.h>

#define MAXAYBUFFER 8192

#include <stdio.h>
#include "aspec.h"


extern Z80Regs spectrumZ80;
extern int gSoundInited;

AYData ay_data[MAXAYBUFFER];	// sound data. Legend: lower part: TState where the out was generated. Bit 18 -> out #fe,0 ->0  out #fe,16 -> 1
int currentAYPtr=-1;		// place in the soundData buffer where we'll store next out #fe

AY38192 ay_chip;
AY38192 last_frame_ay;			// ay as was in last frame
#define PI_VAL 3.1415926535898

unsigned int tone_inc[4096];				// 16:16 fixed value, holds the increment position of the wave for a given tone 

// waveform arrays
int waveform_type_array[16]={0,0,0,0,1,1,1,1,4,0,7,2,5,3,6,1};
int loggy_waveform[16]={0,2,3,4,6,8,12,16,24,32,48,64,96,128,192,255};
byte waveform[8][512];		// 512 bytes for each waveform
unsigned int waveform_period_to_increment[65536];
#define LOG_POWER 2.0f

void initAYarrays (void)
{
int i,j;
float f;


	// init inverse tone (wave position increment)
	tone_inc[0]=0;				// this is an invalid value... I suppose!
	for (i=1;i<4096;i++)
	{
		f=(float)i;

		f=110837.5/f;				// get output frequency
		if (f>22050.0f) f=22050.f;	// frequencies>22050 -> 22050

		f=f/22050.0f;				// get wave increment

		f=f*65536.0*32768.0;		// convert to 1:31 (upper bit will be output)
		tone_inc[i]=(int)f;
	}

	// ********************  create 8 waveforms
#define LOG_WAVEFORM
#ifndef LOG_WAVEFORM
	// waveform 0 : decay & end
	for (i=0;i<256;i++)
	{
		waveform[0][i]=255-i;			//mmm... this should be log
		waveform[0][256+i]=0;
	}

	// waveform 1 : raise & end
	for (i=0;i<256;i++)
	{
		waveform[1][i]=i;			//mmm... this should be log
		waveform[1][256+i]=0;
	}

	// waveform 2 : decay & hold
	for (i=0;i<256;i++)
	{
		waveform[2][i]=255-i;			//mmm... this should be log
		waveform[2][256+i]=255;
	}

	// waveform 3 : raise & hold
	for (i=0;i<256;i++)
	{
		waveform[3][i]=i;			//mmm... this should be log
		waveform[3][256+i]=255;
	}

	// waveform 4 : decay & again
	for (i=0;i<256;i++)
	{
		waveform[4][i]=255-i;			//mmm... this should be log
		waveform[4][256+i]=255-i;
	}

	// waveform 5 : raise & again
	for (i=0;i<256;i++)
	{
		waveform[5][i]=i;			//mmm... this should be log
		waveform[5][256+i]=i;
	}

	// waveform 6 : raise & decay
	for (i=0;i<256;i++)
	{
		waveform[6][i]=i;			//mmm... this should be log
		waveform[6][256+i]=255-i;
	}

	// waveform 7 : decay & raise
	for (i=0;i<256;i++)
	{
		waveform[7][i]=255-i;			//mmm... this should be log
		waveform[7][256+i]=i;
	}
#else
		// waveform 0 : decay & end
	for (i=0;i<256;i++)
	{
		f=(float)255-i;
		f=f/256.0f;
		f=pow(f,LOG_POWER);
		f=f*256.0f;
		waveform[0][i]=(int)f;
		waveform[0][256+i]=0;
	}

	// waveform 1 : raise & end
	for (i=0;i<256;i++)
	{
		f=(float)i;
		f=f/256.0f;
		f=pow(f,LOG_POWER);
		f=f*256.0f;
		waveform[1][i]=(int)f;
		waveform[1][256+i]=0;
	}

	// waveform 2 : decay & hold
	for (i=0;i<256;i++)
	{
		f=(float)255-i;
		f=f/256.0f;
		f=pow(f,LOG_POWER);
		f=f*256.0f;
		waveform[2][i]=(int)f;	
		waveform[2][256+i]=255;
	}

	// waveform 3 : raise & hold
	for (i=0;i<256;i++)
	{
		f=(float)i;
		f=f/256.0f;
		f=pow(f,LOG_POWER);
		f=f*256.0f;
		waveform[3][i]=(int)f;
		waveform[3][256+i]=255;
	}

	// waveform 4 : decay & again
	for (i=0;i<256;i++)
	{
		f=(float)255-i;
		f=f/256.0f;
		f=pow(f,LOG_POWER);
		f=f*256.0f;
		waveform[4][i]=(int)f;
		waveform[4][256+i]=(int)f;
	}

	// waveform 5 : raise & again
	for (i=0;i<256;i++)
	{
		f=(float)i;
		f=f/256.0f;
		f=pow(f,LOG_POWER);
		f=f*256.0f;
		waveform[5][i]=(int)f;
		waveform[5][256+i]=(int)f;
	}

	// waveform 6 : raise & decay
	for (i=0;i<256;i++)
	{
		f=(float)i;
		f=f/256.0f;
		f=pow(f,LOG_POWER);
		f=f*256.0f;
		waveform[6][i]=(int)f;
		f=(float)255-i;
		f=f/256.0f;
		f=pow(f,LOG_POWER);
		f=f*256.0f;
		waveform[6][256+i]=(int)f;
	}

	// waveform 7 : decay & raise
	for (i=0;i<256;i++)
	{
		f=(float)255-i;
		f=f/256.0f;
		f=pow(f,LOG_POWER);
		f=f*256.0f;
		waveform[7][i]=(int)f;		
		f=(float)i;
		f=f/256.0f;
		f=pow(f,LOG_POWER);
		f=f*256.0f;
		waveform[7][256+i]=(float)f;
	}
#endif



	// create waveform_period_to_increment
	waveform_period_to_increment[0]=0;		// invalid value
	for (i=1;i<65536;i++)
	{
		f=(float)i;

		f=6927.34375/f;				// get output frequency
		f=f/44100.0f;
		f=f*32.0f;
		f=f*65536.0*65536.0f;
		waveform_period_to_increment[i]=(int)f;


	}
}

void ayinit (AY38192 *ay)
{
int i;
	for (i=0;i<15;i++) ay->reg[i]=0xff;
	ay->last_address=0;
	for (i=0;i<3;i++) 
	{
		ay->wavepos[i]=0;
		ay->waveform_acc[i]=0;
		ay->waveform_int[i]=0;
	}
}

void ay_init (void)
{
	ayinit(&ay_chip);
	ayinit(&last_frame_ay);


}

void initAYLog (void)
{
	
	if (currentAYPtr==-1)	// first time initialization
	{
		ay_data[0].value=0;
		ay_data[0].port=0;
		ay_data[0].tcount=0;
		currentAYPtr=1;
	}
	else						// at least this is the second frame
	{
		ay_data[0].value=ay_data[currentAYPtr-1].value;
		ay_data[0].port=ay_data[currentAYPtr-1].port;
		ay_data[0].tcount=ay_data[currentAYPtr-1].tcount;
		currentAYPtr=1;

	}


}



void outport_bffd (Z80Regs *regs,byte value)
{
int val;
	if (ay_chip.last_address>13) return;		// don't log invalid ports
	if (currentAYPtr>=MAXAYBUFFER) return;		// no space

	
	ay_data[currentAYPtr].value=value;
	ay_data[currentAYPtr].port=ay_chip.last_address;
	ay_data[currentAYPtr].tcount=spectrumZ80.IPeriod-spectrumZ80.ICount;

	// modify AY chip
	ay_chip.reg[ay_chip.last_address]=value;


	currentAYPtr++;
}

void outport_fffd (Z80Regs *regs,byte value)
{
	// just save the port address
	ay_chip.last_address=value;
}

byte inport_fffd (Z80Regs *regs)
{
	if (ay_chip.last_address>13) return 0xff;
	return ay_chip.reg[ay_chip.last_address];
}



u8 ayTone[3][882];
AY38192 ayW;

// generate tone/noise data (without amplitude control)
// noise is not used for the moment
void generateSoundChannel (int nch)
{
int tone;
int i,j,cur;

int tgt;
float tmp;
float divisor;
u8 *toneData;
unsigned wavepos,increment;

if (currentAYPtr>3) 
	toneData=NULL;

	toneData=ayTone[nch];

	divisor=882.0/((float)spectrumZ80.IPeriod);

	memcpy (&ayW,&last_frame_ay,sizeof(AY38192));
	
	wavepos=ayW.wavepos[nch];

	tone=ayW.reg[nch*2]+(ayW.reg[nch*2+1]&15)*256;		// get starting tone for this channel
	if ((ayW.reg[7]&(1<<nch))) tone=0;
	increment=tone_inc[tone&4095];

	cur=0;						// change to pointer to get better performance
	for (i=1;i<currentAYPtr;i++)
	{
		tgt=ay_data[i].tcount;		// get next change
		tmp=tgt;
		tmp*=divisor;
		tmp+=0.5;
		j=(int)tmp;

		for (;cur<j;cur++) 
		{
			if (wavepos>>31) toneData[cur]=0xff;		// use last bit
			else toneData[cur]=0;
			
			wavepos=wavepos+increment;
		}

		// evolve ayW
		ayW.reg[ay_data[i].port]=ay_data[i].value;

		tone=ayW.reg[nch*2]+(ayW.reg[nch*2+1]&15)*256;		// get new tone
		if ((ayW.reg[7]&(1<<nch))) tone=0;
		increment=tone_inc[tone&4095];
	}

	// end of loop, extend note (or noise)
	for (;cur<882;cur++) 
	{
		if (wavepos>>31) toneData[cur]=0xff;		// use last bit
		else toneData[cur]=0;
		
		wavepos=wavepos+increment;
	}

	// change this channel's wavepos
	last_frame_ay.wavepos[nch]=wavepos;
	ayW.wavepos[nch]=wavepos;


}





void applyWaveform (int nch)
{
int tone;
int i,j,cur;
int tgt;
float tmp;
float divisor;
u8 *toneData;
unsigned int wavepos,increment,period;
int volume,use_waveform;
int curval;
unsigned int waveform_acc,waveform_int,waveform_type;

if (currentAYPtr>3) 
	toneData=NULL;

	toneData=ayTone[nch];

	divisor=882.0/((float)spectrumZ80.IPeriod);

	memcpy (&ayW,&last_frame_ay,sizeof(AY38192));

	volume=ayW.reg[8+nch]&15;
	use_waveform=ayW.reg[8+nch]&16;
	period=ayW.reg[11]+ayW.reg[12]*256;
	increment=waveform_period_to_increment[period];
	waveform_type=waveform_type_array[ayW.reg[13]&15];

	waveform_int=ayW.waveform_int[nch];
	waveform_acc=ayW.waveform_acc[nch];

	cur=0;

	for (i=1;i<currentAYPtr;i++)
	{
		tgt=ay_data[i].tcount;		// get next change
		tmp=tgt;
		tmp*=divisor;
		tmp+=0.5;
		j=(int)tmp;

		for (;cur<j;cur++) 
		{
			curval=toneData[cur];

			if (use_waveform)
			{
				// apply waveform
				waveform_acc+=increment;
				if (waveform_acc&0xe0000000)		// check overflow
				{
					waveform_acc=waveform_acc&0xdfffffff;
					waveform_int++;
					if (waveform_int==512)
					{
						if (waveform_type<4) waveform_int=256;
						else waveform_int=0;
					}
				}

				
				curval=curval*waveform[waveform_type][waveform_int];
				toneData[cur]=curval>>8;		// /256 due to waveform apply

						
			}
			else
			{
				curval=curval*volume;
				toneData[cur]=curval>>4;		// /16 due to volume multiplication
			}

		}

		// evolve ayW
		ayW.reg[ay_data[i].port]=ay_data[i].value;
		if (ay_data[i].port>10) 			// fixme: when do waveforms are reseted?
		{
			waveform_int=0;
			waveform_acc=0;
		}


		// paste from initialization
		volume=ayW.reg[8+nch]&7;
		use_waveform=ayW.reg[8+nch]&8;
		period=ayW.reg[11]+ayW.reg[12]*256;
		increment=waveform_period_to_increment[period];
		waveform_type=waveform_type_array[ayW.reg[13]&15];

	}
	
	for (;cur<882;cur++) 
	{
		curval=toneData[cur];

		if (use_waveform)
		{
			// apply waveform
			waveform_acc+=increment;
			if (waveform_acc&0xe0000000)		// check overflow
			{
				waveform_acc=waveform_acc&0xdfffffff;
				waveform_int++;
				if (waveform_int==512)
				{
					if (waveform_type<4) waveform_int=256;
					else waveform_int=0;
				}
			}

			
			curval=curval*waveform[waveform_type][waveform_int];
			toneData[cur]=curval>>8;		// /256 due to waveform apply

					
		}
		else
		{
			curval=curval*volume;
			toneData[cur]=curval>>4;		// /16 due to volume multiplication
		}

	}


	ayW.waveform_int[nch]=waveform_int;
	ayW.waveform_acc[nch]=waveform_acc;
	last_frame_ay.waveform_int[nch]=waveform_int;
	last_frame_ay.waveform_acc[nch]=waveform_acc;


}


int f=0;
void AYDump (void)
{
int i,j,k;
u8 *ptr;
int ch;

//#define SINGLE_CHANNEL
#ifdef SINGLE_CHANNEL
	ch=0;


	generateSoundChannel (ch);
	applyWaveform (ch);

	ptr=gGetSampleBuffer(1);
	for (j=0;j<882;j++) 
	{
		ptr[j]=ayTone[ch][j];			// 8 upper bits only
	}
	gPlaySound (1);

	initAYLog();
	memcpy (&last_frame_ay,&ayW,sizeof(AY38192));

#else

for (k=0;k<3;k++)
{

	generateSoundChannel (k);
	applyWaveform (k);



	ptr=gGetSampleBuffer(k+1);
	for (j=0;j<882;j++) 
	{
		ptr[j]=ayTone[k][j];			// 8 upper bits only
	}
	gPlaySound (k+1);
}
	initAYLog();
	memcpy (&last_frame_ay,&ayW,sizeof(AY38192));
#endif

}




