#include "stdafx.h"
#define MAXSOUNDBUFFER 8192

#include <stdio.h>
#include "aspec.h"


extern Z80Regs spectrumZ80;
extern int gSoundInited;

int soundData[MAXSOUNDBUFFER];	// sound data. Legend: lower part: TState where the out was generated. Bit 18 -> out #fe,0 ->0  out #fe,16 -> 1
int currentSoundPtr=-1;		// place in the soundData buffer where we'll store next out #fe


// initialize port #fe logging
void initSoundLog (void)
{
	if (currentSoundPtr==-1)	// first time initialization
	{
		soundData[0]=0;
		currentSoundPtr=1;
	}
	else						// at least this is the second frame
	{
		soundData[0]=soundData[currentSoundPtr-1]&262144;	// keep last's frame sound status
		currentSoundPtr=1;
	}
}


// logSound logs sound data for an entire t-state cycle. Once finished
// sound is pushed to the Audio Card (latency = 1/50th)
void logSound (int state)
{
int val;

	if (currentSoundPtr>=MAXSOUNDBUFFER) return;		// no space, no log (should never happen, buffer is big enough)
	if (state>0) state=262144;		// 0 or 16 -> 0 or 262144

	if ((soundData[currentSoundPtr-1]&262144)==state) return;	// no need to log if it's the same state than before

	val=spectrumZ80.IPeriod-spectrumZ80.ICount;		// get current tstate
	val+=state;										// add him beeper state (should be NOT (previous state)  )

	soundData[currentSoundPtr]=val;				// store
	currentSoundPtr++;

}


// builds a sound buffer "frame"
void soundDump (void)
{
	if (!gSoundInited) return;				// no init -> no sound -> bye

	beeperDump();			// disabled for the moment
	AYDump();

}

void beeperDump (void)
{
int cur_sound,tgt;
int i,j,cur;
float tmp;
u8 *beeperSound;
FILE *f;
float divisor;



	divisor=882.0/((float)spectrumZ80.IPeriod);

	// create sound, 44100, mono, 8 bits unsigned

	cur_sound=soundData[0]&262144;			// get current beeper status
	if (cur_sound) cur_sound=255;			// convert it to 0 or 255 (unsigned sound data)
	else cur_sound=0;

	cur=0;							// change to pointer to get better performance
	beeperSound=gGetSampleBuffer(0);			// get the place to put the new sound
if (beeperSound!=NULL)
{

	for (i=1;i<currentSoundPtr;i++)			// for each logged port data
	{
		tgt=soundData[i]&262143;			// get tsate
		tmp=tgt;
		tmp*=divisor;						// scale to 0..882
		tmp+=0.5;
		j=(int)tmp;

		for (;cur<j;cur++) beeperSound[cur]=cur_sound;		// fill the buffer with current status until a new out #fe appears

		cur_sound=soundData[i]&262144;		// get next status
		if (cur_sound) cur_sound=255;
		else cur_sound=0;
	}

	// end filling all the buffer till its end
	for (;cur<882;cur++) beeperSound[cur]=cur_sound;

	gPlaySound(0);		// let the music play :) (waits til last buffer was played)
}
	initSoundLog();		// reset sound logging

}

