#include "stdafx.h"

#include "debug2.h"
#include "aspec/aspec.h"


// tmp
#include "zxDEB.h"
#include "zxDEBDlg.h"


DebugLineData debugData[DEB_LINES];
DebugLineData *currentLine;			// line we're working (so that the struct can be filled by other functions)
int lastPC=0;	

int plain_DLD=0;

// breakpoints
u8 breakpointRAM[MAX_RAM_BANKS][16384];		
u8 breakpointROM[MAX_ROM_BANKS][16384];		

extern int ZXDEB_break;
char PCcomment[1024];
extern int selectedPC;
extern OpData *currentOp;

specialBreakpoint specialBKP[MAX_SPECIAL_BKP];
int numSpecialBkps=0;
int specialBkp=0;			// disable 

DebugLineData debug2_dld;

void exitDebug (void)
{
	free_slots ();
}

void ZXDEBinitialization(void)
{
int i,j;
	for (j=0;j<MAX_RAM_BANKS;j++)
	for (i=0;i<16384;i++) breakpointRAM[j][i]=0;

	for (j=0;j<MAX_ROM_BANKS;j++)
	for (i=0;i<16384;i++) breakpointROM[j][i]=0;
}

int spacePad (char *txt,int count)
{
int len;

	len=strlen(txt);
	if (len>count)			// too much text in the array: fix this
	{
		strcat (txt,"   ");
		return 3;
	}

	while (len<count)
	{
		txt[len]=' ';
		len++;
	}
	txt[count]=0;
	return 0;

}



char tmpNumber[128];
// prints a number in the specified format (hex, dec, hex + h , $ + hex, ...)
char *printDebugNumber (int num)
{
	sprintf (tmpNumber,"%04x",num);
	return tmpNumber;
}

char *printDebugNumber8bits (int num)
{
	sprintf (tmpNumber,"%02x",num);
	return tmpNumber;
}


// given a flag, checks if it's correct (with current processor status)
int checkFlags (int flag)
{
Z80Regs *regs;
	regs=&spectrumZ80;

	if (flag==COND_NONE) return 1;
	if (flag==COND_C) 
	{
		if (TEST_FLAG(FLAG_C)) return 1;
		return 0;
	}

	if (flag==COND_NC) 
	{
		if (TEST_FLAG(FLAG_C)) return 0;
		return 1;
	}

	if (flag==COND_Z)
	{
		if (TEST_FLAG(FLAG_Z)) return 1;
		return 0;
	}

	if (flag==COND_NZ)
	{
		if (TEST_FLAG(FLAG_Z)) return 0;
		return 1;
	}
	return 1;
}


DebugLineData tmpLine;
int instructionSize (int pc)
{
char txt[128];
int tmp;

	currentLine=&tmpLine;
	tmp=getDebugData (pc,txt);
	return tmp-pc;
}

int instructionType (int pc)
{
char txt[128];
	currentLine=&tmpLine;
	getDebugData (pc,txt);
	return currentOp->inst;
}



void debug2_set_bkp (int pc,int bank)
{
	if (bank<32760)
	{
		if (breakpointRAM[bank][pc]==1) breakpointRAM[bank][pc]=0;
		else breakpointRAM[bank][pc]=1;
	}
	else
	{
		if (breakpointROM[bank-32768][pc]==1) breakpointROM[bank-32768][pc]=0;
		else breakpointROM[bank-32768][pc]=1;
	}
}

void debug2_set_bkp (int pc)
{
int bank;

	bank=spectrumZ80.pagenum[pc>>14];
	debug2_set_bkp(pc&16383,bank);
}


// given a z80-space pc, returns if there's a breakpoint there
int there_s_a_breakpoint_at (int pc)
{
	if (pc<16384)
	{
		if (breakpointROM[pc>>14][pc&16383]) return 1;
		return 0;
	}

	if (breakpointRAM[spectrumZ80.pagenum[pc>>14]][pc&16383]) return 1;
	return 0;

}

eword *getRegister (REGs reg,DebugLineData *d)
{
	if (reg==REG_HL) return &spectrumZ80.HL;
	if (reg==REG_BC) return &spectrumZ80.BC;
	if (reg==REG_DE) return &spectrumZ80.DE;
	if (reg==REG_SP) return &spectrumZ80.SP;
	if (reg==REG_IXY) 
	{
		if (d->ixy==1) return &spectrumZ80.IX;
		else return &spectrumZ80.IY;
	}
	return NULL;
}

void debug2_step_into (void)
{
	Z80Run (&spectrumZ80,1);
}

int debug2_check_special_bkps (void)
{
int i;
int min,max;
eword *ew;
int val;

for (i=0;i<numSpecialBkps;i++)
{
	min=specialBKP[i].min_val;
	max=specialBKP[i].max_val;
	switch (specialBKP[i].event_type)
	{
		case (event_in):
							if (debug2_dld.opcode->inst==INST_IN)
								if (debug2_dld.parameterData>=min && debug2_dld.parameterData<=max)
									return 1;
		case (event_out):
							if (debug2_dld.opcode->inst==INST_OUT)
								if (debug2_dld.parameterData>=min && debug2_dld.parameterData<=max)
									return 1;
		case (event_memread):
							if (debug2_dld.opcode->inst==INST_MEMREAD)
							{
								if (debug2_dld.opcode->regs==REG_UNUSED)	// then it's NN
								{
									if (debug2_dld.parameterData>=min && debug2_dld.parameterData<=max)
										return 1;
								}
								else
								{
									// it's a register
									ew=getRegister (debug2_dld.opcode->regs,&debug2_dld);
									if (ew!=NULL) 
									{
										val=ew->W;
										val+=debug2_dld.offset;
										if (val>=min && val<=max) return 1;
									}
								}
							}
							break;

		case (event_memwrite):
							if (debug2_dld.opcode->inst==INST_MEMWRITE)
							{
								if (debug2_dld.opcode->regs==REG_UNUSED)	// then it's NN
								{
									if (debug2_dld.parameterData>=min && debug2_dld.parameterData<=max)
										return 1;
								}
								else
								{
									// it's a register
									ew=getRegister (debug2_dld.opcode->regs,&debug2_dld);
									if (ew!=NULL) 
									{
										val=ew->W;
										val+=debug2_dld.offset;
										if (val>=min && val<=max) return 1;
									}
								}
							}
							break;

		default: break;
	}
}
	return 0;
}

extern int stepThroughBkp,stepThroughBkpBank;
extern int gSoundInited;


extern CZxDEBDlg *mainWindow;
void debug2_run_with_bkp (void)
{
int tstate,oldpc;
int pcval;
static int last_sample_pos=0,current_sample_pos;

	ZXDEB_break=0;
	while (1)
	{
		if (specialBkp) 
		{
			plain_DLD=1;			// disable big comments & labels
			fillDebuggerLines(&debug2_dld,spectrumZ80.PC.W,0,1);
			plain_DLD=0;
			if (debug2_check_special_bkps()) break;
		}
	
		tstate=spectrumZ80.ICount;
		Z80Run (&spectrumZ80,1);
		if (tstate<spectrumZ80.ICount)	// refresh needed
		{

			screenRedraw_forZXDEB ();

			if (gSoundInited)
			{
				gSoundSync();
				soundDump ();
			}
			// else find a way to sync without sound (timer)


			// read keys, once per frame
			keyboardHandler_forZXDEB ();
			if (ZXDEB_break) break;

		}

		if (!specialBkp)		// if special bkp, then normal breakpoints can't stop the game
		if (there_s_a_breakpoint_at(spectrumZ80.PC.W)) break;


	}


	if (stepThroughBkp!=-1)
	{
		if (stepThroughBkpBank<32760) breakpointRAM[stepThroughBkpBank][stepThroughBkp]-=2;
		else breakpointROM[stepThroughBkpBank-32768][stepThroughBkp]-=2;
		stepThroughBkp=-1;
	}



//	gClearSoundBuffers ();			// configurable option 

}


int debug2_save_debug_info (char *file)
{
FILE *f;
	f=fopen (file,"wb");
	if (f==NULL) return 0;

	save_text_data (f);
	fclose (f);
	return 1;
}



int debug2_load_debug_info (char *file)
{
FILE *f;

	f=fopen (file,"rt");
	if (f==NULL) return 0;

	load_text_data(f);
	return 1;
}


extern int GDD_ixy,GDD_offset;


// fills debugLines array to show what's going on in the debugger
void fillDebuggerLinesFrom (DebugLineData *array,int first_instr,int NUMLINES)
{
int line;
char txt[128],comment[1024];
int pc,oldpc,tmp;
int tabber;
char *ptr;
TFD *tfd;
int status;

	line=0;
	pc=first_instr;
	PCcomment[0]=0;
	status=0;
	while (1)
	{
		currentLine=&(array[line]);


		// search for BEFORE comments
		// search for labels
if (!plain_DLD)
{
		tfd=searchLabel (pc);
		if (tfd!=NULL)
		{
			sprintf (currentLine->line,"%s:",tfd->txt);
			currentLine->pc=-2;

			line++;
			if (line==NUMLINES) break;
			currentLine=&(array[line]);
		}
}
		tabber=0;
		oldpc=pc;
		pc=getDebugData (pc,txt);
		if (oldpc==65536)
		{ 
			line++; 
			if (line==NUMLINES) break;
			continue; 
		}
		// check now if pc>comment index (when it's there)



		currentLine->jump=0;
		currentLine->ixy=GDD_ixy;
		currentLine->offset=GDD_offset;

		// build text data

		// PC
		sprintf (array[line].line,"%s",printDebugNumber(oldpc));
		
		tabber+=6;
		tabber+=spacePad (array[line].line,tabber);		// add spaces till character [tabber]

		// opcode here if you want...

		// instruction
		sprintf (array[line].line+tabber,"%s",txt);
		array[line].pc=oldpc;


		tabber+=25;
		tabber+=spacePad (array[line].line,tabber);


		// current instruction? -> then we can add extra data
		if (oldpc==spectrumZ80.PC.W)
		{
		 if (checkFlags (currentLine->opcode->cond))
		 {
			if (currentLine->opcode->inst==INST_CALL ||
				currentLine->opcode->inst==INST_JUMP ||
				currentLine->opcode->inst==INST_RET) 
					currentLine->jump=1;
		 }

		 if (currentLine->opcode->inst==INST_DJNZ)
		 {
			 if (spectrumZ80.BC.B.h!=1) currentLine->jump=1;
		 }


		}


		tabber+=spacePad (array[line].line,tabber);

		// comment here (if needed)
		tfd=searchComment (oldpc);
		if (tfd!=NULL)
		{
			sprintf (comment,"; %s",tfd->txt);
			comment[128-tabber-1]=0;		// too much chars for the line
			strcat (array[line].line,comment);
			tmp=selectedPC;
			if (tmp==-1) tmp=spectrumZ80.PC.W;
			if (oldpc==tmp) strcpy (PCcomment,tfd->txt);
		}
		


		line++;
		if (line==NUMLINES) break;

	}
}


void fillDebuggerLines (DebugLineData *array,int first_instr,int lock,int top)
{
int i;

	if (!lock)
	{
		fillDebuggerLinesFrom (array,first_instr,top);
		lastPC=first_instr;
		return;
	}

	fillDebuggerLinesFrom (array,lastPC,top);

	// look if you see the lines from the last position
	for (i=0;i<top;i++)		
	{
		if (array[i].pc==first_instr) break;
	}

	if (i!=top) return;		// it's seen, so its ok

	
	fillDebuggerLinesFrom (array,first_instr,top);	// it's not seen, so redraw
	lastPC=first_instr;
}

