#include "stdafx.h"


#include "debug2.h"
#include "aspec/aspec.h"


extern Z80Regs spectrumZ80;
#undef Z80ReadMem
#define Z80ReadMem(x) Z80MemRead ((x),&spectrumZ80)
extern DebugLineData *currentLine;
extern int showLabels;

textForDebugger *firstText=NULL;
void readline (FILE *f,char *dst)
{
char ch;
	while (1)
	{
		fread (&ch,1,1,f);
		if (feof(f)) break;
		if (ch==13) continue;
		if (ch==10) break;
		*dst=ch;
		dst++;
	}
	*dst=0;
}


void save_text_data (FILE *f)
{
TFD *tmp;
	tmp=firstText;

	for (tmp=firstText;tmp!=NULL;tmp=(TFD *)tmp->next)
	{
		fprintf (f,"TEXTDATA %d %d %d START %s\n",tmp->bank,tmp->pc,tmp->type,tmp->txt);
	}

}

void free_tfd (TFD *tfd)
{
	free (tfd->txt);
}

void free_text_data (void)
{
TFD *ptr,*tmp;
CString str;
	
	ptr=firstText;
	while (ptr!=NULL)
	{
		tmp=(TFD *)ptr->next;
		free_tfd (ptr);
		free (ptr);
		ptr=tmp;
	}
	firstText=NULL;
}

void load_text_data (FILE *f)
{
char line[2048];
char txt1[128],txt2[128];
int v1,v2,v0;
TFD *txt,*cur;
char *ptr;
	
	if (firstText!=NULL) free_text_data();
	

	firstText=NULL;

	while (1)
	{
		readline (f,line);
		if (strlen(line)<8) break;		// at least must have TEXTDATA

		sscanf (line,"%s %d %d %d %s",txt1,&v0,&v1,&v2,txt2);		// get first parameters + first word of comment
		if (strcmp(txt1,"TEXTDATA")!=0) continue;
		if (strcmp(txt2,"START")!=0) continue;
		ptr=strstr(line,"START")+6;

		// fix only to allow old file format
		if (v0==-1)
		{
			v0=spectrumZ80.pagenum[v1>>14];
			v1=v1&16383;
		}

		txt=allocNewText (ptr,v0,v1,v2);

		if (feof(f)) break;

	}
}

TFD *allocNewText (char *txt,int pc,int type)
{
int bank;
	bank=spectrumZ80.pagenum[pc>>14];
	pc=pc&16383;
	return allocNewText (txt,bank,pc,type);
}

TFD *allocNewText (char *txt,int bank,int pc,int type)
{
TFD *tmp,*tmp2;

	// get mem
	tmp=(TFD *)malloc (sizeof(textForDebugger));
	if (tmp==NULL) return NULL;

	// create data
	tmp->txt=strdup(txt);
	tmp->pc=pc;
	tmp->bank=bank;
	tmp->type=type;
	tmp->next=NULL;

	// concatenate
	if (firstText==NULL) firstText=tmp;
	else
	{
		tmp2=firstText;
		while (tmp2->next!=NULL)
		{
			tmp2=(TFD *)tmp2->next;
		}
		tmp2->next=tmp;
	}

	return tmp;

}


void deleteEntry (TFD *tfd)
{
TFD *tmp,*last=NULL;
	for (tmp=firstText;tmp!=NULL;last=tmp,tmp=(TFD *)tmp->next)
	{
		if (tmp==tfd)
		{
			// delete item from chain
			if (last==NULL) firstText=(TFD *)tmp->next;
			else last->next=tmp->next;

			// free data
			free (tmp->txt);
			free (tmp);
			break;
		}
	}
}


void modifyText (TFD *tfd,char *comment)
{
	free (tfd->txt);
	tfd->txt=strdup(comment);
}

TFD *genericSearch (int pc,int typ)
{
TFD *tmp;
int bank;

	tmp=firstText;

	bank=spectrumZ80.pagenum[pc>>14];
	pc=pc&16383;

	for (tmp=firstText;tmp!=NULL;tmp=(TFD *)tmp->next)
	{
		if (tmp->pc!=pc) continue;
		if (tmp->bank!=bank) continue;
		if (tmp->type!=typ) continue;
		return tmp;
	}

	return NULL;
}

TFD *searchComment (int pc)
{
	return genericSearch (pc,0);
}

TFD *searchLabel (int pc)
{
	return genericSearch (pc,1);
}

char tmpLabelText[256];
char *printLabel (int pc)
{
TFD *tmp;
int bank;

	if (!showLabels) 
	{
		sprintf (tmpLabelText,"%s",printDebugNumber(pc));
		return tmpLabelText;
	}

	tmp=searchLabel(pc);
	if (tmp==NULL) sprintf (tmpLabelText,"%s",printDebugNumber(pc));
	else sprintf (tmpLabelText,"%s",tmp->txt);

	return tmpLabelText;
}


TFD *searchLabelByText (const char *txt)
{
TFD *tmp;

	tmp=firstText;
	for (;tmp!=NULL;tmp=(TFD*)tmp->next)
	{
		if (tmp->type==TYPE_LABEL)
			if (strcmp (tmp->txt,txt)==0) return tmp;
	}
	return NULL;

}
